#!/usr/bin/perl -w

### cleanup.pl


unlink "corpus_vocab_db.dir";                                             
unlink "corpus_vocab_db.pag";                                             
unlink "doc_vectors_db";                                                  
unlink "normalized_doc_vecs_db"; 

